/*
	Gforecast.c
	Roberto Machorro <rmach@ciberlinux.net>

	A weather forecast providing GNOME applet.
*/

#include <gnome.h>
#include <applet-widget.h>
#include <sys/wait.h>

#include "choose.xpm"
#include "update.xpm"

#define __WEATHER_SITE		"http://www.weather.com/weather/local/"
#define __FORECAST_STRING	"http:\\/\\/image.weather.com\\/web\\/common\\/wxicons\\/"
#define __BROWSER		"gnome-moz-remote --newwin \"%s\""

gboolean post_update = TRUE;
guint timer_id = -1;
gdouble refresh_time = 60.0;
gchar *weather_city_file = NULL;
GtkWidget *image_widget;
gchar *tmp_file_name = NULL;
gboolean first_instance = FALSE;
gint applet_size;
GtkWidget *city_file, *refresh;

static void refresh_image();

void show_info_msg(gchar *msg)
{
/*	GtkWidget *dlg = gnome_message_box_new(msg, GNOME_MESSAGE_BOX_INFO,
				GNOME_STOCK_BUTTON_OK, NULL);
	gnome_dialog_run_and_close(GNOME_DIALOG(dlg));
*/
	gchar *nmsg;
	nmsg = g_strconcat(msg, "\n", NULL);
	g_print(nmsg);
	g_free(nmsg);
}

void show_error_msg(gchar *msg)
{
	GtkWidget *dlg = gnome_message_box_new(msg, GNOME_MESSAGE_BOX_ERROR,
				GNOME_STOCK_BUTTON_OK, NULL);
	gnome_dialog_run_and_close(GNOME_DIALOG(dlg));
}

void fetch_forecast_image_url(char *img_url)
{
	FILE *ph;
	gchar *str_url = g_strconcat(__WEATHER_SITE, weather_city_file, NULL);
	gchar *str_cmd = g_strdup_printf(
		"wget -q -O - %s|sed -n '/%s/!d;s/wxicons\\/31/wxicons\\/52/p;tquit;bnext;:quit;q;:next'|awk -F'\"' '{print $2}'",
		str_url, __FORECAST_STRING);
	ph = popen(str_cmd, "r");
	if (ph != NULL) {
		if (fgets(img_url, 255, ph) == NULL) {
			strcpy(img_url, "");
		}
		pclose(ph);
	} else {
		strcpy(img_url, "");
	}
	g_free(str_cmd);
	g_free(str_url);
}

static gint applet_save_session(GtkWidget *widget,
				const char *private_config_path,
				const char *global_config_path)
{
	gnome_config_push_prefix(private_config_path);
	gnome_config_set_float("config/refresh", refresh_time);
	gnome_config_set_string("config/cityfile", weather_city_file);
	gnome_config_pop_prefix();

	gnome_config_sync();
	gnome_config_drop_all();

	return FALSE;
}

void load_image(gchar *file_name)
{
	if (file_name == NULL) {
		if (!first_instance)
			gnome_pixmap_load_xpm_d_at_size(GNOME_PIXMAP(image_widget),
					update_xpm, applet_size, applet_size);
	} else {
		gnome_pixmap_load_file_at_size(GNOME_PIXMAP(image_widget),
					file_name, applet_size, applet_size);
	}
}

static void applet_change_size(GtkWidget *widget, int size, gpointer data)
{
	applet_size = size;
	if (post_update) {
		load_image(NULL);
	} else {
		load_image(tmp_file_name);
	}
}

static gint timer_callback(gpointer data)
{
	refresh_image();
	return TRUE;
}

static void update_callback(AppletWidget *widget, gpointer data)
{
	refresh_image();
}

static void refresh_image()
{
	gchar *img_cmd = NULL;
	char img_url[255];
	GDK_THREADS_ENTER();
	first_instance = FALSE;
	gtk_timeout_remove(timer_id);
	if (post_update) {
		post_update = FALSE;
		/* baja la imagen y la muestra */
		show_info_msg(_("Getting image URL..."));
		fetch_forecast_image_url(img_url);
		show_info_msg(_("Got image URL."));
		if (strlen(img_url) < 2) {
			show_error_msg(_("Could not get forecast, wrong file maybe?"));
		} else {
			if (tmp_file_name == NULL) {
				tmp_file_name = tempnam("/tmp", ".Gf-");
			}
			img_cmd = g_strdup_printf("wget -q -O %s %s",
							tmp_file_name, img_url);
			show_info_msg(_("Getting image..."));
			system(img_cmd);
			g_free(img_cmd);
			show_info_msg(_("Got image."));
			load_image(tmp_file_name);
			timer_id = gtk_timeout_add(60000*refresh_time,
							timer_callback, NULL);
		}
	} else {
		show_info_msg(_("Showing question mark."));
		post_update = TRUE;
		load_image(NULL);
		timer_id = gtk_timeout_add(1000, timer_callback, NULL);
	}
	GDK_THREADS_LEAVE();
}

static void apply_config(GnomeDialog *dlg, gint button, gpointer data)
{
	if (button == 0) {
		gchar *file;
		file = gtk_entry_get_text(GTK_ENTRY(city_file));
		if (weather_city_file != NULL)
			g_free(weather_city_file);
		weather_city_file = g_strdup(file);
		refresh_time = (double)atof(
				gtk_entry_get_text(GTK_ENTRY(refresh)));
		post_update = FALSE;
		show_info_msg(_("Syncing config."));
		applet_widget_sync_config(APPLET_WIDGET(data));
		gtk_widget_hide(GTK_WIDGET(dlg));
		timer_callback(NULL);
	}
}

static void configuration_dialog(AppletWidget *applet, gpointer data)
{
	GtkWidget *file_holder, *refresh_holder, *label;
	GtkWidget *dlg;
	gchar *buffer;

	dlg = gnome_dialog_new(_("Configuration"), GNOME_STOCK_BUTTON_OK,
					GNOME_STOCK_BUTTON_CANCEL, NULL);
	gnome_dialog_set_default(GNOME_DIALOG(dlg), 0);

	refresh_holder = gtk_hbox_new(FALSE, 0);
	label = gtk_label_new(_("Refresh (mins.):"));
	gtk_box_pack_start(GTK_BOX(refresh_holder), label, TRUE, FALSE, 0);
	gtk_widget_show(label);
	refresh = gtk_entry_new();
	buffer = g_strdup_printf("%f", refresh_time);
	gtk_entry_set_text(GTK_ENTRY(refresh), buffer);
	g_free(buffer);
	gtk_box_pack_start(GTK_BOX(refresh_holder), refresh, TRUE, TRUE, 0);
	gtk_widget_show(refresh);
	gtk_widget_show(refresh_holder);

	file_holder = gtk_hbox_new(FALSE, 0);
	label = gtk_label_new(_("City HTML file:"));
	gtk_box_pack_start(GTK_BOX(file_holder), label, TRUE, FALSE, 0);
	gtk_widget_show(label);
	city_file = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(city_file), weather_city_file);
	gtk_box_pack_start(GTK_BOX(file_holder), city_file, TRUE, TRUE, 0);
	gtk_widget_show(city_file);
	gtk_widget_show(file_holder);

	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dlg)->vbox), refresh_holder,
					TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dlg)->vbox), file_holder,
					TRUE, TRUE, 0);

	gtk_signal_connect(GTK_OBJECT(dlg), "clicked",
				GTK_SIGNAL_FUNC(apply_config), data);

	gnome_dialog_run_and_close(GNOME_DIALOG(dlg));
}

static void launch_forecast_website(AppletWidget *applet, gpointer data)
{
	if (weather_city_file != NULL) {
		gchar *url = g_strconcat(__WEATHER_SITE, weather_city_file, NULL);
		gchar *cmd = g_strdup_printf(__BROWSER, url);

		if(!WIFEXITED(system(cmd))) {
			show_error_msg(_("Failed to launch browser"));
		}

		g_free(cmd);
		g_free(url);
	}
}

static void about_dialog(AppletWidget *applet, gpointer data)
{
	const char *authors[] = {"Roberto Machorro <rmach@bigfoot.com>",
				"Tomasz Kloczko <kloczek@pld.org.pl>", NULL};
	GtkWidget *dlg = gnome_about_new(_("Gforecast"), VERSION,
				"Copyright (C) 2000-2001 Roberto Machorro",
				authors, "This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or  (at your option) any later version.",
				NULL);
	gtk_widget_show(dlg);
}

int main(int argc, char *argv[])
{
	GtkWidget *applet;

	applet_widget_init("Gforecast", VERSION, argc, argv, NULL, 0, NULL);

	applet = applet_widget_new("Gforecast");

	if (!applet) {
		g_error("Could not create applet!\n");
	}

	gnome_config_push_prefix(APPLET_WIDGET(applet)->privcfgpath);
	refresh_time = gnome_config_get_float("config/refresh");
	weather_city_file = gnome_config_get_string("config/cityfile");
	gnome_config_pop_prefix();
	if (weather_city_file == NULL) {
		first_instance = TRUE;
		show_info_msg(_("Default city is Ensenada, Mexico."));
		refresh_time = 60.0;
		weather_city_file = _("MXXX0046");
	}

	applet_size = applet_widget_get_panel_pixel_size(APPLET_WIDGET(applet));
	image_widget = gnome_pixmap_new_from_xpm_d_at_size(choose_xpm,
				applet_size, applet_size);
	applet_widget_add(APPLET_WIDGET(applet), image_widget);
	gtk_widget_show(image_widget);

	gtk_widget_show(applet);

	gtk_signal_connect(GTK_OBJECT(applet), "save_session",
				GTK_SIGNAL_FUNC(applet_save_session),
				NULL);

	gtk_signal_connect(GTK_OBJECT(applet), "change_pixel_size",
				GTK_SIGNAL_FUNC(applet_change_size),
				NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
					"Config",
					GNOME_STOCK_MENU_PROP,
					_("Properties"),
					configuration_dialog,
					applet);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
					"Website",
					GNOME_STOCK_MENU_EXEC,
					_("View Website"),
					launch_forecast_website,
					NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
					"Update",
					GNOME_STOCK_MENU_REFRESH,
					_("Update"),
					update_callback,
					NULL);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
					"About",
					GNOME_STOCK_MENU_ABOUT,
					_("About"),
					about_dialog,
					NULL);

	if (!first_instance) {
		timer_id = gtk_timeout_add(1000, timer_callback, NULL);
	}

	applet_widget_gtk_main();

	return 0;
}
